@@ -1,7 +1,7 @@
 // drumkv1_jack.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2018, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -631,6 +631,28 @@ void drumkv1_jack::updateSample (void)
 #endif
 
 
+#ifdef HAVE_SIGNAL_H
+
+#include <QSocketNotifier>
+
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <signal.h>
+
+// File descriptor for SIGTERM notifier.
+static int g_fdSigterm[2] = { -1, -1 };
+
+// Unix SIGTERM signal handler.
+void drumkv1_sigterm_handler ( int /*signo*/ )
+{
+	char c = 1;
+
+	(::write(g_fdSigterm[0], &c, sizeof(c)) > 0);
+}
+
+#endif	// HAVE_SIGNAL_H
+
+
 // Constructor.
 drumkv1_jack_application::drumkv1_jack_application ( int& argc, char **argv )
 	: QObject(NULL), m_pApp(NULL), m_bGui(true),
@@ -655,12 +677,52 @@ drumkv1_jack_application::drumkv1_jack_application ( int& argc, char **argv )
 		m_pApp = new QApplication(argc, argv);
 	else
 		m_pApp = new QCoreApplication(argc, argv);
+
+#ifdef HAVE_SIGNAL_H
+
+	// set to ignore any fatal "Broken pipe" signals.
+	::signal(SIGPIPE, SIG_IGN);
+
+	// initialize file descriptors for SIGTERM socket notifier.
+	::socketpair(AF_UNIX, SOCK_STREAM, 0, g_fdSigterm);
+	m_pSigtermNotifier = new QSocketNotifier(
+		g_fdSigterm[1], QSocketNotifier::Read, this);
+
+	QObject::connect(m_pSigtermNotifier,
+		SIGNAL(activated(int)),
+		SLOT(sigterm_handler()));
+
+	// install SIGTERM signal handler.
+	struct sigaction term;
+	term.sa_handler = drumkv1_sigterm_handler;
+	::sigemptyset(&term.sa_mask);
+	term.sa_flags = 0;
+	term.sa_flags |= SA_RESTART;
+	::sigaction(SIGTERM, &term, NULL);
+	::sigaction(SIGINT,  &term, NULL);
+
+	// ignore SIGHUP signal.
+	struct sigaction hup;
+	hup.sa_handler = SIG_IGN;
+	::sigemptyset(&hup.sa_mask);
+	hup.sa_flags = 0;
+	hup.sa_flags |= SA_RESTART;
+	::sigaction(SIGHUP, &hup, NULL);
+
+#else
+
+	m_pSigtermNotifier = NULL;
+
+#endif	// !HAVE_SIGNAL_H
 }
 
 
 // Destructor.
 drumkv1_jack_application::~drumkv1_jack_application (void)
 {
+#ifdef HAVE_SIGNAL_H
+	if (m_pSigtermNotifier) delete m_pSigtermNotifier;
+#endif
 #ifdef CONFIG_NSM
 	if (m_pNsmClient) delete m_pNsmClient;
 #endif
@@ -720,7 +782,7 @@ bool drumkv1_jack_application::setup (void)
 
 	if (m_bGui) {
 		m_pWidget = new drumkv1widget_jack(m_pDrumk);
-		m_pWidget->show();
+	//	m_pWidget->show();
 		if (m_presets.isEmpty())
 			m_pWidget->initPreset();
 		else
@@ -755,7 +817,10 @@ bool drumkv1_jack_application::setup (void)
 		if (m_pWidget)
 			m_pWidget->setNsmClient(m_pNsmClient);
 	}
+	else
 #endif	// CONFIG_NSM
+	if (m_pWidget)
+		m_pWidget->show();
 
 	return true;
 }
@@ -799,7 +864,9 @@ void drumkv1_jack_application::openSession (void)
 	if (!dir.exists())
 		dir.mkpath(path_name);
 
-	const QFileInfo fi(path_name, display_name + '.' + DRUMKV1_TITLE);
+	QFileInfo fi(path_name, "session." DRUMKV1_TITLE);
+	if (!fi.exists())
+		fi.setFile(path_name, display_name + '.' + DRUMKV1_TITLE);
 	if (fi.exists()) {
 		const QString& sFilename = fi.absoluteFilePath();
 		if (m_pWidget) {
@@ -831,10 +898,11 @@ void drumkv1_jack_application::saveSession (void)
 	qDebug("drumkv1_jack::saveSession()");
 #endif
 
-	const QString& path_name = m_pNsmClient->path_name();
-	const QString& display_name = m_pNsmClient->display_name();
 //	const QString& client_id = m_pNsmClient->client_id();
-	const QFileInfo fi(path_name, display_name + '.' + DRUMKV1_TITLE);
+	const QString& path_name = m_pNsmClient->path_name();
+//	const QString& display_name = m_pNsmClient->display_name();
+//	const QFileInfo fi(path_name, display_name + '.' + DRUMKV1_TITLE);
+	const QFileInfo fi(path_name, "session." DRUMKV1_TITLE);
 
 	drumkv1_param::savePreset(m_pDrumk, fi.absoluteFilePath(), true);
 
@@ -882,6 +950,22 @@ void drumkv1_jack_application::hideSession (void)
 #endif	// CONFIG_NSM
 
 
+#ifdef HAVE_SIGNAL_H
+
+// SIGTERM signal handler.
+void drumkv1_jack_application::sigterm_handler (void)
+{
+	char c;
+
+	if (::read(g_fdSigterm[1], &c, sizeof(c)) > 0) {
+		if (m_pApp)
+			m_pApp->quit();
+	}
+}
+
+#endif	// HAVE_SIGNAL_H
+
+
 //-------------------------------------------------------------------------
 // main
 
