@@ -24,6 +24,10 @@
 #include "drumkv1_config.h"
 #include "drumkv1_sample.h"
 
+#include "drumkv1_ui.h"
+
+#include "drumkv1widget_spinbox.h"
+
 #include <sndfile.h>
 
 #include <QPainter>
@@ -47,7 +51,7 @@
 // Constructor.
 drumkv1widget_sample::drumkv1widget_sample (
 	QWidget *pParent, Qt::WindowFlags wflags )
-	: QFrame(pParent, wflags), m_pSample(0), m_iChannels(0), m_ppPolyg(0)
+	: QFrame(pParent, wflags), m_pSample(NULL), m_iChannels(0), m_ppPolyg(NULL)
 {
 	QFrame::setMouseTracking(true);
 	QFrame::setFocusPolicy(Qt::ClickFocus);
@@ -61,8 +65,8 @@ drumkv1widget_sample::drumkv1widget_sample (
 	QFrame::setFrameShape(QFrame::Panel);
 	QFrame::setFrameShadow(QFrame::Sunken);
 
-	m_bLoop = false;
-	m_iLoopStart = m_iLoopEnd = 0;
+	m_bOffset = false;
+	m_iOffsetStart = m_iOffsetEnd = 0;
 
 	m_dragCursor  = DragNone;
 	m_pDragSample = NULL;
@@ -91,6 +95,9 @@ void drumkv1widget_sample::setSample ( drumkv1_sample *pSample )
 
 	m_pSample = pSample;
 
+//	m_bOffset = 0;
+//	m_iOffsetStart = m_iOffsetEnd = 0;
+
 	m_pDragSample = NULL;
 
 	if (m_pSample)
@@ -139,6 +146,7 @@ void drumkv1widget_sample::setSample ( drumkv1_sample *pSample )
 	update();
 }
 
+
 drumkv1_sample *drumkv1widget_sample::sample (void) const
 {
 	return m_pSample;
@@ -158,45 +166,46 @@ const QString& drumkv1widget_sample::sampleName (void) const
 }
 
 
-void drumkv1widget_sample::setLoop ( bool bLoop )
+// Offset mode.
+void drumkv1widget_sample::setOffset ( bool bOffset )
 {
-	m_bLoop = bLoop;
+	m_bOffset = bOffset;
 
 	updateToolTip();
 	update();
 }
 
-bool drumkv1widget_sample::isLoop (void) const
+bool drumkv1widget_sample::isOffset (void) const
 {
-	return m_bLoop;
+	return m_bOffset;
 }
 
 
-void drumkv1widget_sample::setLoopStart ( uint32_t iLoopStart )
+void drumkv1widget_sample::setOffsetStart ( uint32_t iOffsetStart )
 {
-	m_iLoopStart = iLoopStart;
+	m_iOffsetStart = iOffsetStart;
 
 	updateToolTip();
 	update();
 }
 
-uint32_t drumkv1widget_sample::loopStart (void) const
+uint32_t drumkv1widget_sample::offsetStart (void) const
 {
-	return m_iLoopStart;
+	return m_iOffsetStart;
 }
 
 
-void drumkv1widget_sample::setLoopEnd ( uint32_t iLoopEnd )
+void drumkv1widget_sample::setOffsetEnd ( uint32_t iOffsetEnd )
 {
-	m_iLoopEnd = iLoopEnd;
+	m_iOffsetEnd = iOffsetEnd;
 
 	updateToolTip();
 	update();
 }
 
-uint32_t drumkv1widget_sample::loopEnd (void) const
+uint32_t drumkv1widget_sample::offsetEnd (void) const
 {
-	return m_iLoopEnd;
+	return m_iOffsetEnd;
 }
 
 
@@ -228,13 +237,13 @@ void drumkv1widget_sample::mousePressEvent ( QMouseEvent *pMouseEvent )
 		if (m_dragCursor == DragNone) {
 			m_dragState = DragStart;
 			m_posDrag = pMouseEvent->pos();
-		} else if (m_bLoop) {
-			const int w = QFrame::width();
+		} else {
 			const uint32_t nframes = m_pSample->length();
-			if (nframes > 0) {
-				m_iDragStartX = safeX((m_iLoopStart * w) / nframes);
-				m_iDragEndX   = safeX((m_iLoopEnd   * w) / nframes);
-				m_dragState   = m_dragCursor;
+			if (nframes > 0 && m_bOffset) {
+				const int w = QFrame::width();
+				m_iDragOffsetStartX = safeX((m_iOffsetStart * w) / nframes);
+				m_iDragOffsetEndX   = safeX((m_iOffsetEnd   * w) / nframes);
+				m_dragState = m_dragCursor;
 			}
 		}
 	}
@@ -248,95 +257,111 @@ void drumkv1widget_sample::mouseMoveEvent ( QMouseEvent *pMouseEvent )
 	const int x = pMouseEvent->pos().x();
 
 	switch (m_dragState) {
-	case DragNone: {
-		if (m_bLoop && m_pSample) {
-			const int w = QFrame::width();
+	case DragNone:
+		if (m_pSample) {
 			const uint32_t nframes = m_pSample->length();
 			if (nframes > 0) {
-				const int x1 = (m_iLoopStart * w) / nframes;
-				const int x2 = (m_iLoopEnd   * w) / nframes;
-				if (abs(x1 - x) < QApplication::startDragDistance()) {
-					m_dragCursor = DragLoopStart;
+				const int w  = QFrame::width();
+				const int dx = QApplication::startDragDistance();
+				const int x1 = (m_iOffsetStart * w) / nframes;
+				const int x2 = (m_iOffsetEnd   * w) / nframes;
+				if (abs(x2 - x) < dx && m_bOffset) {
+					m_dragCursor = DragOffsetEnd;
 					QFrame::setCursor(QCursor(Qt::SizeHorCursor));
+					QToolTip::showText(
+						QCursor::pos(),
+						tr("Offset end: %1")
+							.arg(textFromValue(m_iOffsetEnd)), this);
 				}
 				else
-				if (abs(x2 - x) < QApplication::startDragDistance()) {
-					m_dragCursor = DragLoopEnd;
+				if (abs(x1 - x) < dx && m_bOffset) {
+					m_dragCursor = DragOffsetStart;
 					QFrame::setCursor(QCursor(Qt::SizeHorCursor));
+					QToolTip::showText(
+						QCursor::pos(),
+						tr("Offset start: %1")
+							.arg(textFromValue(m_iOffsetStart)), this);
 				}
 				else
 				if (m_dragCursor != DragNone) {
-					m_dragCursor =  DragNone;
+					m_dragCursor  = DragNone;
 					QFrame::unsetCursor();
 				}
 			}
 		}
 		break;
-	}
-	case DragLoopStart: {
-		if (m_iDragEndX > x) {
-			m_iDragStartX = safeX(x);
-			update();
-			if (m_pSample) {
-				const int w = QFrame::width();
-				if (w > 0) {
-					const uint32_t nframes = m_pSample->length();
-					QToolTip::showText(
-						QCursor::pos(),
-						tr("Loop start: %1")
-							.arg((m_iDragStartX * nframes) / w), this);
-				}
-			}
-		}
-		break;
-	}
-	case DragLoopEnd: {
-		if (m_iDragStartX < x) {
-			m_iDragEndX = safeX(x);
-			update();
-			if (m_pSample) {
-				const int w = QFrame::width();
-				if (w > 0) {
-					const uint32_t nframes = m_pSample->length();
-					QToolTip::showText(
-						QCursor::pos(),
-						tr("Loop end: %1")
-							.arg((m_iDragEndX * nframes) / w), this);
-				}
-			}
-		}
-		break;
-	}
-	case DragSelect: {
-		// Rubber-band selection...
-		const QRect& rect = QRect(m_posDrag, pMouseEvent->pos()).normalized();
-		m_iDragStartX = safeX(rect.left());
-		m_iDragEndX   = safeX(rect.right());
-		update();
+	case DragOffsetStart:
 		if (m_pSample) {
+			m_iDragOffsetStartX = safeX(x);
+			if (m_iDragOffsetStartX > m_iDragOffsetEndX)
+				m_iDragOffsetStartX = m_iDragOffsetEndX;
+			update();
 			const int w = QFrame::width();
 			if (w > 0) {
 				const uint32_t nframes = m_pSample->length();
+				const uint32_t iOffsetStart
+					= (m_iDragOffsetStartX * nframes) / w;
 				QToolTip::showText(
 					QCursor::pos(),
-					tr("Loop start: %1, end: %2")
-						.arg((m_iDragStartX * nframes) / w)
-						.arg((m_iDragEndX   * nframes) / w), this);
+					tr("Offset start: %1")
+						.arg(textFromValue(iOffsetStart)), this);
+			}
+		}
+		break;
+	case DragOffsetEnd:
+		if (m_pSample) {
+			m_iDragOffsetEndX = safeX(x);
+			if (m_iDragOffsetEndX < m_iDragOffsetStartX)
+				m_iDragOffsetEndX = m_iDragOffsetStartX;
+			update();
+			const int w = QFrame::width();
+			if (w > 0) {
+				const uint32_t nframes = m_pSample->length();
+				const uint32_t iOffsetEnd
+					= (m_iDragOffsetEndX * nframes) / w;
+				QToolTip::showText(
+					QCursor::pos(),
+					tr("Offset end: %1")
+						.arg(textFromValue(iOffsetEnd)), this);
+			}
+		}
+		break;
+	case DragOffsetRange:
+		// Rubber-band offset selection...
+		if (m_pSample) {
+			const QRect& rect = QRect(m_posDrag, pMouseEvent->pos()).normalized();
+			m_iDragOffsetStartX = safeX(rect.left());
+			m_iDragOffsetEndX   = safeX(rect.right());
+			update();
+			const int w = QFrame::width();
+			if (w > 0) {
+				const uint32_t nframes      = m_pSample->length();
+				const uint32_t iOffsetStart = (m_iDragOffsetStartX * nframes) / w;
+				const uint32_t iOffsetEnd   = (m_iDragOffsetEndX   * nframes) / w;
+				QToolTip::showText(
+					QCursor::pos(),
+					tr("Offset start: %1, end: %2")
+						.arg(textFromValue(iOffsetStart))
+						.arg(textFromValue(iOffsetEnd)), this);
 			}
 		}
 		break;
-	}
 	case DragStart:
 		// Rubber-band starting...
 		if ((m_posDrag - pMouseEvent->pos()).manhattanLength()
 			> QApplication::startDragDistance()) {
 			// Start dragging alright...
-			if (m_bLoop && (pMouseEvent->modifiers()
+			if (m_dragCursor != DragNone)
+				m_dragState = m_dragCursor;
+			else
+			if (m_bOffset && (pMouseEvent->modifiers()
 				& (Qt::ShiftModifier | Qt::ControlModifier))) {
-				m_dragState = m_dragCursor = DragSelect;
-				m_iDragStartX = m_iDragEndX = m_posDrag.x();
+				m_dragState = m_dragCursor = DragOffsetRange;
+				m_iDragOffsetStartX = m_iDragOffsetEndX = m_posDrag.x();
 				QFrame::setCursor(QCursor(Qt::SizeHorCursor));
-			} else if (m_pSample && m_pSample->filename()) {
+			}
+			else
+			if (m_pSample && m_pSample->filename()) {
 				QList<QUrl> urls;
 				m_pDragSample = m_pSample;
 				urls.append(QUrl::fromLocalFile(m_pDragSample->filename()));
@@ -362,35 +387,35 @@ void drumkv1widget_sample::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 	QFrame::mouseReleaseEvent(pMouseEvent);
 
 	switch (m_dragState) {
-	case DragSelect: {
+	case DragOffsetStart: {
 		const int w = QFrame::width();
 		if (m_pSample && w > 0) {
 			const uint32_t nframes = m_pSample->length();
-			m_iLoopStart = (m_iDragStartX * nframes) / w;
-			m_iLoopEnd   = (m_iDragEndX   * nframes) / w;
-			emit loopRangeChanged();
+			m_iOffsetStart = (m_iDragOffsetStartX * nframes) / w;
+			emit offsetRangeChanged();
 			updateToolTip();
 			update();
 		}
 		break;
 	}
-	case DragLoopStart: {
+	case DragOffsetEnd: {
 		const int w = QFrame::width();
 		if (m_pSample && w > 0) {
 			const uint32_t nframes = m_pSample->length();
-			m_iLoopStart = (m_iDragStartX * nframes) / w;
-			emit loopRangeChanged();
+			m_iOffsetEnd = (m_iDragOffsetEndX * nframes) / w;
+			emit offsetRangeChanged();
 			updateToolTip();
 			update();
 		}
 		break;
 	}
-	case DragLoopEnd: {
+	case DragOffsetRange: {
 		const int w = QFrame::width();
 		if (m_pSample && w > 0) {
 			const uint32_t nframes = m_pSample->length();
-			m_iLoopEnd = (m_iDragEndX * nframes) / w;
-			emit loopRangeChanged();
+			m_iOffsetStart = (m_iDragOffsetStartX * nframes) / w;
+			m_iOffsetEnd   = (m_iDragOffsetEndX   * nframes) / w;
+			emit offsetRangeChanged();
 			updateToolTip();
 			update();
 		}
@@ -460,7 +485,7 @@ void drumkv1widget_sample::resetDragState (void)
 	if (m_dragCursor != DragNone)
 		QFrame::unsetCursor();
 
-	m_iDragStartX = m_iDragEndX = 0;
+	m_iDragOffsetStartX = m_iDragOffsetEndX = 0;
 
 	m_dragState = m_dragCursor = DragNone;
 }
@@ -484,29 +509,11 @@ void drumkv1widget_sample::paintEvent ( QPaintEvent *pPaintEvent )
     painter.fillRect(rect, rgbDark);
 
 	if (m_pSample && m_ppPolyg) {
+		const bool bEnabled = isEnabled();
+		const uint32_t nframes = m_pSample->length();
 		const int w2 = (w << 1);
 		painter.setRenderHint(QPainter::Antialiasing, true);
-		if (m_bLoop && isEnabled()) {
-			int x1, x2;
-			if (m_iDragStartX < m_iDragEndX) {
-				x1 = m_iDragStartX;
-				x2 = m_iDragEndX;
-			} else {
-				const uint32_t nframes = m_pSample->length();
-				if (nframes > 0) {
-					x1 = (m_iLoopStart * w) / nframes;
-					x2 = (m_iLoopEnd   * w) / nframes;
-				}
-				else x1 = x2 = 0;
-			}
-			QLinearGradient grad1(0, 0, w2, h);
-			painter.setPen(pal.highlight().color());
-			grad1.setColorAt(0.0f, rgbLite.darker());
-			grad1.setColorAt(0.5f, pal.dark().color());
-			painter.fillRect(x1, 0, x2 - x1, h, grad1);
-			painter.drawLine(x1, 0, x1, h);
-			painter.drawLine(x2, 0, x2, h);
-		}
+		// Sample waveform...
 		QLinearGradient grad(0, 0, w2, h);
 		painter.setPen(bDark ? Qt::gray : Qt::darkGray);
 		grad.setColorAt(0.0f, rgbLite);
@@ -514,6 +521,38 @@ void drumkv1widget_sample::paintEvent ( QPaintEvent *pPaintEvent )
 		painter.setBrush(grad);
 		for (unsigned short k = 0; k < m_iChannels; ++k)
 			painter.drawPolygon(*m_ppPolyg[k]);
+		// Offset line...
+		if (m_bOffset && bEnabled) {
+			int x1 = 0, x2 = 0;
+			if (m_dragState == DragOffsetStart ||
+				m_dragState == DragOffsetEnd   ||
+				m_dragState == DragOffsetRange) {
+				x1 = m_iDragOffsetStartX;
+				x2 = m_iDragOffsetEndX;
+			}
+			else
+			if (nframes > 0) {
+				x1 = (m_iOffsetStart * w) / nframes;
+				x2 = (m_iOffsetEnd   * w) / nframes;
+			}
+			QColor rgbOver = rgbDark.darker();
+			rgbOver.setAlpha(120);
+			painter.setPen(rgbLite.darker(160));
+			painter.setBrush(rgbDark.lighter(160));
+			QPolygon polyg(3);
+		//	polyg.putPoints(0, 3, x1 + 8, 0, x1, 8, x1, 0);
+		//	painter.drawPolygon(polyg);
+			polyg.putPoints(0, 3, x1 + 8, h, x1, h - 8, x1, h);
+			painter.drawPolygon(polyg);
+			painter.fillRect(0, 0, x1, h, rgbOver);
+			painter.drawLine(x1, 0, x1, h);
+			polyg.putPoints(0, 3, x2 - 8, 0, x2, 8, x2, 0);
+			painter.drawPolygon(polyg);
+		//	polyg.putPoints(0, 3, x2 - 8, h, x2, h - 8, x2, h);
+		//	painter.drawPolygon(polyg);
+			painter.fillRect(x2, 0, w, h, rgbOver);
+			painter.drawLine(x2, 0, x2, h);
+		}
 		painter.setRenderHint(QPainter::Antialiasing, false);
 	} else {
 		painter.setPen(pal.midlight().color());
@@ -620,6 +659,29 @@ void drumkv1widget_sample::loadSample ( drumkv1_sample *pSample )
 }
 
 
+// Value/text format converter utilities.
+uint32_t drumkv1widget_sample::valueFromText ( const QString& text ) const
+{
+	drumkv1widget_spinbox::Format format = drumkv1widget_spinbox::Frames;
+	drumkv1_config *pConfig = drumkv1_config::getInstance();
+	if (pConfig)
+		format = drumkv1widget_spinbox::Format(pConfig->iFrameTimeFormat);
+	const float srate = (m_pSample ? m_pSample->sampleRate() : 44100.0f);
+	return drumkv1widget_spinbox::valueFromText(text, format, srate);
+}
+
+
+QString drumkv1widget_sample::textFromValue ( uint32_t value ) const
+{
+	drumkv1widget_spinbox::Format format = drumkv1widget_spinbox::Frames;
+	drumkv1_config *pConfig = drumkv1_config::getInstance();
+	if (pConfig)
+		format = drumkv1widget_spinbox::Format(pConfig->iFrameTimeFormat);
+	const float srate = (m_pSample ? m_pSample->sampleRate() : 44100.0f);
+	return drumkv1widget_spinbox::textFromValue(value, format, srate);
+}
+
+
 // Update tool-tip.
 void drumkv1widget_sample::updateToolTip (void)
 {
@@ -638,11 +700,11 @@ void drumkv1widget_sample::updateToolTip (void)
 			.arg(m_pSample->rate());
 	}
 
-	if (m_bLoop && m_iLoopStart < m_iLoopEnd) {
+	if (m_bOffset && m_iOffsetStart < m_iOffsetEnd) {
 		if (!sToolTip.isEmpty()) sToolTip += '\n';
-		sToolTip += tr("Loop start: %1, end: %2")
-			.arg(m_iLoopStart)
-			.arg(m_iLoopEnd);
+		sToolTip += tr("Offset start: %1, end: %2")
+			.arg(textFromValue(m_iOffsetStart))
+			.arg(textFromValue(m_iOffsetEnd));
 	}
 
 	setToolTip(sToolTip);
